"""
This is a simple wrapper around lru_cache which allows
us to enable caching only within some scope defined by contextmanager

It makes it easier to cache some functions e.g. only inside wpos-user get
scope and still be able to get fresh data in wpos-user enable
where cached commands might return different results because of outside
"""
from contextlib import contextmanager
from functools import lru_cache

_CACHED_FUNCTIONS = {}
CACHING_ENABLED = False


@contextmanager
def enable_caching():
    """
    This context manager sets global marker
    which enables caching
    """
    global CACHING_ENABLED
    if CACHING_ENABLED:
        raise RuntimeError('nested scopes are not supported')

    try:
        CACHING_ENABLED = True
        yield
    finally:
        _clear_cache()
        CACHING_ENABLED = False


def _clear_cache():
    """
    Clear cache of all registered functions
    """
    for func in _CACHED_FUNCTIONS.values():
        func.cache_clear()


def cached_in_scope(func):
    """
    Primary code which decides should we return cached
    result or fresh
    """
    def _wrapper(*args, **kwargs):
        if CACHING_ENABLED:
            return _CACHED_FUNCTIONS[func.__name__](*args, **kwargs)
        return func(*args, **kwargs)

    cached_func = lru_cache()(func)
    _CACHED_FUNCTIONS[func.__name__] = cached_func
    return _wrapper
